MODULE SPI;
(* =========================================================================  
   Astrobe Library Functions for STM32F7 SPI0/1 
   
   Oberon Version (c) 2017 CFB Software   
   http://www.astrobe.com  
   
   ========================================================================= *)

IMPORT GPIO, MCU, SYSTEM;

CONST
  SPI1* = 1;
  SPI3* = 3;
  TXE  = 1;
  RXNE = 0;
  
  CLKDIV2*   = 0;
  CLKDIV4*   = 1;
  CLKDIV8*   = 2;
  CLKDIV16*  = 3;
  CLKDIV32*  = 4;
  CLKDIV64*  = 5;
  CLKDIV128* = 6;
  CLKDIV256* = 7;
  
TYPE 
  ConfigurePinsProc* = PROCEDURE;
  
VAR
  CR1 : INTEGER;
  CR2 : INTEGER;
  DR  : INTEGER;
  SR  : INTEGER;
  SSPCPSR : INTEGER;
  I2SCFGR : INTEGER;
  response: BYTE;

PROCEDURE* Response*(): BYTE;
BEGIN
  RETURN response
END Response;


PROCEDURE* SendByte*(data: BYTE);
BEGIN
  SYSTEM.PUT(DR, data);
  REPEAT UNTIL SYSTEM.BIT(SR, RXNE);
  SYSTEM.GET(DR, response)
END SendByte;  


PROCEDURE* SendChar*(data: CHAR);
BEGIN
  SYSTEM.PUT(DR, data);
  REPEAT UNTIL SYSTEM.BIT(SR, RXNE);
  SYSTEM.GET(DR, response) 
END SendChar;  


PROCEDURE* SendData*(data: INTEGER);
BEGIN
  SYSTEM.PUT(DR, data);
  REPEAT UNTIL SYSTEM.BIT(SR, RXNE);
  SYSTEM.GET(DR, response) 
END SendData;  


PROCEDURE* Send*(buf: ARRAY OF BYTE; length: INTEGER);
VAR
  i, data: INTEGER; 
BEGIN 
  ASSERT(length <= LEN(buf), 20); 
  FOR i := 0 TO length - 1 DO 
    SYSTEM.PUT(DR, buf[i]);
    REPEAT UNTIL SYSTEM.BIT(SR, TXE); 
    SYSTEM.GET(DR, response) 
  END
END Send; 
 

PROCEDURE* ReceiveByte*(): BYTE; 
VAR
  data: BYTE; 
BEGIN 
  SYSTEM.PUT(DR, 0FFH);
  REPEAT UNTIL SYSTEM.BIT(SR, RXNE); 
  SYSTEM.GET(DR, data); 
  RETURN data 
END ReceiveByte; 
 

PROCEDURE* ReceiveChar*(): CHAR; 
VAR
  data: CHAR; 
BEGIN 
  SYSTEM.PUT(DR, 0FFH);
  REPEAT UNTIL SYSTEM.BIT(SR, RXNE); 
  SYSTEM.GET(DR, data); 
  RETURN data 
END ReceiveChar; 
 

PROCEDURE* ReceiveData*(): INTEGER; 
VAR
  data: INTEGER; 
BEGIN 
  SYSTEM.PUT(DR, 0FFH);
  REPEAT UNTIL SYSTEM.BIT(SR, RXNE); 
  SYSTEM.GET(DR, data); 
  RETURN data 
END ReceiveData; 
 

PROCEDURE* Receive*(VAR buf: ARRAY OF BYTE; length: INTEGER);
VAR
  i: INTEGER;
BEGIN
  ASSERT(length <= LEN(buf), 21); 
  FOR i := 0 TO length - 1 DO 
    SYSTEM.PUT(DR, 0FFH);
    REPEAT UNTIL SYSTEM.BIT(SR, RXNE); 
    SYSTEM.GET(DR, buf[i]) 
  END
END Receive; 


PROCEDURE* SetBus(bus: INTEGER);
VAR
  SPIBase: INTEGER;
BEGIN
  IF bus = SPI1 THEN 
    SPIBase := MCU.SPI1Base
  ELSIF bus = SPI3 THEN
    SPIBase := MCU.SPI3Base
  END;
  CR1  := SPIBase;
  CR2  := SPIBase + 004H;
  SR   := SPIBase + 008H;
  DR   := SPIBase + 00CH;
  I2SCFGR := SPIBase + 01CH;
END SetBus; 
   

PROCEDURE Init*(bus, clkDiv, nBits: INTEGER; ConfigurePins: ConfigurePinsProc);
CONST
  (* CR1 Bits *)
  CPHA     = 0;
  CPOL     = 1;
  MSTR     = 2;
  (* prescale - fPCLK/4 = 27 Mhz *)
  SPE      = 6;
  LSBFIRST = 7;
  SSM      = 9;
  RXONLY   = 10;
  BIDIMODE = 15;
  BIDIOE   = 14;
  (* CR2 Bits *)
  SSOE  = 2;
  NSSP  = 3;
  FRF   = 4;
  FRXTH = 12;
VAR
  cr1, cr2: SET;
  test : BYTE;
  br: SET;
BEGIN
  ASSERT(nBits IN {3..15}, 20);
  ASSERT(bus IN {SPI1, SPI3}, 21);
  SetBus(bus);
  
  (* Select SPI registers *)
  ConfigurePins();

  (* Clock Rate *)
  ASSERT(clkDiv IN {CLKDIV2 .. CLKDIV256}, 22);
  br := BITS(LSL(clkDiv, 3));
  
  (* Master mode, LSB First, Prescale, *)
  (* disabled,not receive-only, bidirectional,  *)
  (* SSM = 0, CPOL=0, CPHA=0 *)
  SYSTEM.PUT(CR1, {MSTR} + br - {SPE, LSBFIRST, RXONLY, CPOL, CPHA, BIDIMODE, BIDIOE}); 
  
  (* Disable I2S *)
  SYSTEM.PUT(I2SCFGR , {} );   

  SYSTEM.GET(CR2, cr2); 
  (* DS = nbits - 1 *)
  SYSTEM.BFI(ORD(cr2), 11, 8, nBits - 1); 
  (* Fifo reception threshold for 8 bits, SS Pulse enabled - NSSP = 1, SSOE = 1, FRF = 0 (SPI mode) *)    
  cr2 := cr2 + {FRXTH, NSSP, SSOE} - {FRF};
  SYSTEM.PUT(CR2, cr2); 
  
  (* Enable SPI*)
  SYSTEM.GET(CR1, cr1);
  SYSTEM.PUT(CR1, cr1 + {SPE})
END Init;

END SPI.

