MODULE ST7735;
(* =========================================================================  
   Example ARM Cortex-M7 Oberon Module  
 
   Description:
     Sitronix ST7735 Display Controller Driver for 128x160 TFT display

   Target:
     STM32F7 systems
     
   Tested on: 
     ARM mbed + AdaFruit 1.8" TFT Display
   
   Reference:
     Sitronix ST7735 262K Color Single-Chip TFT Controller/Driver
     v2.1 2010-02-01
     
   (c) 2015-2017 CFB Software
   http://www.astrobe.com
   
   ========================================================================= *)
IMPORT GPIO, Timers, MCU, SPI, SYSTEM;

CONST 
  SLPOUT  = 011H; (* Sleep out & booster on *)
  CASET   = 02AH; (* Column address set *)
  RASET   = 02BH; (* Row address set *)
  RAMWR   = 02CH; (* Writing to memory *)
  MADCTL  = 036H; (* Memory data access control *)
  COLMOD  = 03AH; (* Colour mode - interface pixel format *)
  DISPON  = 029H; (* Display on *)
  
  (* 16-bit RRRRR-GGGGGG-BBBBB color definitions *)
  Black*   = 00000H;
  Red*     = 0F800H;
  Green*   = 007E0H;
  Blue*    = 0001FH;
  White*   = Red + Green + Blue;
  Cyan*    = Green + Blue;
  Magenta* = Red + Blue;
  Yellow*  = Red + Green;
  
  MaxX* = 159;
  MaxY* = 127;
  
  BackLightPin = 4;  (* PB4 *)
  ResetPin     = 15; (* PA15 *)
  DCPin        = 7;  (* PC7 *)
  Data = TRUE;
  Command = FALSE;
  
VAR
  timer: Timers.Timer;
  
  (* ========================================================================= *) 
  PROCEDURE BackLight(state: BOOLEAN);
  BEGIN
    GPIO.Set(MCU.GPIOBBase, BackLightPin, state)
  END BackLight;
  
  (* ========================================================================= *) 
  PROCEDURE Reset(state: BOOLEAN);
  BEGIN
    GPIO.Set(MCU.GPIOABase, ResetPin, state)
  END Reset;
  
  (* ========================================================================= *) 
  PROCEDURE SetDataType(dataType: BOOLEAN);
  BEGIN
    GPIO.Set(MCU.GPIOCBase, DCPin, dataType)
  END SetDataType;
  
  (* ========================================================================= *) 
  PROCEDURE SendByte(data: BYTE);
  BEGIN
    SPI.SendByte(data)
  END SendByte;  
  
  (* ========================================================================= *) 
  PROCEDURE SendWord(data: INTEGER);
  BEGIN 
    SPI.SendByte(ASR(data, 24));
    SPI.SendByte(ASR(data, 16));
    SPI.SendByte(ASR(data, 8));
    SPI.SendByte(data);
  END SendWord; 
  
  (* ========================================================================= *) 
  PROCEDURE SendCommand(data: BYTE);
  BEGIN
    SetDataType(Command);
    SPI.SendByte(data);
    SetDataType(Data)
  END SendCommand;

  (* ========================================================================= *) 
  PROCEDURE SetXY(x, y: INTEGER);
  BEGIN
    (* Column address *)
    SendCommand(CASET);
    SendWord(LSL(y, 16) + y);
    SendCommand(RASET); 
    SendWord(LSL(x, 16) + x);
    (* Subsequent data will be written to the display *)
    SendCommand(RAMWR) 
  END SetXY;
  
 (* ************************************************************************************* *)
 (* Draws a dot in the specified color at the specified x and y coordinates *)
 (* color = 16-bit color value rrrrrggggggbbbbb *)
  PROCEDURE DrawDot*(colour, x, y: INTEGER);
  BEGIN
    IF (x <= MaxX) & (y <= MaxY) & (x >= 0) & (y >= 0) THEN 
      (* Column address *)
      SetXY(x, y);
      SendByte(colour DIV 256);
      SendByte(colour MOD 256)
    END 
  END DrawDot;

  (* ========================================================================= *) 
  PROCEDURE DrawRectangle*(colour, x, y, width, length: INTEGER);
  VAR
    i, count, colourWord: INTEGER;
  BEGIN
    SendCommand(CASET); 
    SendWord(LSL(y, 16) + y + length - 1); 
    SendCommand(RASET); 
    SendWord(LSL(x, 16) + x + width - 1); 
    SendCommand(RAMWR);
    count := (width * length) DIV 2;
    colourWord := LSL(colour, 16) + colour;
    FOR i := 0 TO count - 1 DO 
      SendWord(colourWord)
    END 
  END DrawRectangle;

  (* ========================================================================= *) 
  PROCEDURE ClearScreen*(colour: INTEGER);
  BEGIN
    DrawRectangle(colour, 0, 0, MaxX + 1, MaxY + 1)
  END ClearScreen;
  
  (* ========================================================================= *) 
  (*  Nucleo      Display
      ----        -------
      AVDD        VCC
      GND         Gnd 
      PA15   D20  RESET
      PC7    D21  D/C
      PB5    D22  MOSI1
      PB3    D23  SCK1
      PA4    D24  CS 
      PB4    D25  LITE
  *)

  (* ========================================================================= *) 
  PROCEDURE ConfigureGPIO;
  VAR
    config: GPIO.PortConfiguration;
    ahbenr: SET;
  BEGIN
    (* enable clock for GPIO-A, B and C *)
    SYSTEM.GET(MCU.RCC_AHB1ENR, ahbenr);
    SYSTEM.PUT(MCU.RCC_AHB1ENR, ahbenr + 
      {MCU.RCC_AHB1ENR_GPIOA} +
      {MCU.RCC_AHB1ENR_GPIOB} +
      {MCU.RCC_AHB1ENR_GPIOC}
      );
    config.mode := GPIO.Mode_Out;
    config.outputType := GPIO.Push_Pull;
    config.speed := GPIO.Speed_50MHz;
    config.resistors := GPIO.Floating;
    config.alternateFunction := GPIO.AF0;
    GPIO.Configure(MCU.GPIOABase, ResetPin, config);
    GPIO.Configure(MCU.GPIOBBase, BackLightPin, config);
    GPIO.Configure(MCU.GPIOCBase, DCPin, config);
    
  END ConfigureGPIO;

  (* ========================================================================= *) 
  PROCEDURE ConfigureSPI3*;
  VAR
    apbenr, ahbenr: SET;
    config: GPIO.PortConfiguration;
  BEGIN
    (* enable SPI3 *)
    SYSTEM.GET(MCU.RCC_APB1ENR, apbenr);
    SYSTEM.PUT(MCU.RCC_APB1ENR, apbenr + {MCU.RCC_APB1ENR_SPI3, MCU.RCC_APB1ENR_PWR});
    
    (* enable GPIOs for SPI *)
    SYSTEM.GET(MCU.RCC_AHB1ENR, ahbenr);          
    SYSTEM.PUT(MCU.RCC_AHB1ENR, ahbenr + 
      {MCU.RCC_AHB1ENR_GPIOA} +
      {MCU.RCC_AHB1ENR_GPIOB});

    (* configure GPIOs *)
    config.mode := GPIO.Mode_AF;
    config.outputType := GPIO.Push_Pull; (* OK *)
    config.speed := GPIO.Speed_50MHz; 
    config.resistors := GPIO.Floating; 
    
    config.alternateFunction := GPIO.AF5;      (* for SPI1 ? *)
    config.alternateFunction := GPIO.AF6;      (* for SPI3 ? *)
    
    GPIO.Configure(MCU.GPIOBBase, 3, config);  (* B3, SPI SCK  *)
    GPIO.Configure(MCU.GPIOABase, 4, config);  (* A4  SPI SS   *)
    GPIO.Configure(MCU.GPIOBBase, 5, config)   (* B5  SPI MOSI *)
  END ConfigureSPI3;
  

  (* ========================================================================= *) 
  (* Initialise the Sirtronix ST7735 TFT Controller *)
  PROCEDURE Init*();
  CONST
    nbits = 8;
  BEGIN
    ConfigureGPIO();
    
    Timers.Init(timer, Timers.TIM2, Timers.MSecs);
    SPI.Init(SPI.SPI3, SPI.CLKDIV2, nbits, ConfigureSPI3); 

    Reset(FALSE);
    SetDataType(Command);
    Reset(TRUE);
    Timers.Delay(timer, 120); 
    SendCommand(SLPOUT); 
    Timers.Delay(timer, 120); 
    SendCommand(COLMOD); 
    SendByte(05H);   (* RGB16 5-6-5 *)
    SendCommand(MADCTL); (* Portrait RGB *)
    SendByte(046H); 
    SendCommand(DISPON); 
    BackLight(TRUE)
  END Init;
  
END ST7735.
